#include <Python.h>
#include <vector>
#define OBGREP_MODULE
#include "OBGrep.h"
#include "OBGrep_wrapper.h"
#include <string>
#include <iostream.h> //cin,cout,cerr

PyObject *wrap_loadDB(PyObject *self, PyObject *args)
{
  char *inFileName;
  int  returnValue;

  if (!PyArg_ParseTuple(args, "s", &inFileName))
    return NULL;
  returnValue = loadDB(inFileName);
  return Py_BuildValue( "i" ,returnValue);
}

PyObject *wrap_match(PyObject *self, PyObject *args)
{
  char *pattern;
  int  dbNum;
  int  returnValue;
  bool scan1, scan2;

  if (!(PyArg_ParseTuple(args, "is", &dbNum, &pattern)))
    return NULL;
  returnValue = match(dbNum,pattern);
  return Py_BuildValue( "i" ,returnValue);
}

PyObject *wrap_getNumEntries(PyObject *self, PyObject *args)
{
  int  dbNum;
  int  returnValue;

  if (!PyArg_ParseTuple(args, "i", &dbNum))
    return NULL;
  returnValue = getNumEntries(dbNum);
  return Py_BuildValue( "i" ,returnValue);
}

PyObject *wrap_getEntryString(PyObject *self, PyObject *args)
{
  int  dbNum;
  int  entryNum;
  int  ensure = 20;
  bool correct = true;
  std::string  returnValue;
  PyObject *returnObject;

  if (!PyArg_ParseTuple(args, "ii", &dbNum, &entryNum))
    return NULL;
  returnValue = getEntryString(dbNum,entryNum);
  //cerr << "wrapper:returnValue:\t" << returnValue  <<"\n" ;  
  returnObject = PyString_FromString (returnValue.c_str());
  return returnObject;

}

PyObject *wrap_printEntry(PyObject *self, PyObject *args)
{
  int  dbNum;
  int  entryNum;
  

  if (!PyArg_ParseTuple(args, "ii", &dbNum, &entryNum))
    return NULL;
  printEntry(dbNum,entryNum);
  return Py_None;
}

PyObject *wrap_printDB(PyObject *self, PyObject *args)
{
  int  dbNum;
  

  if (!PyArg_ParseTuple(args, "i", &dbNum))
    return NULL;
  printDB(dbNum);
  return Py_None;
}

PyObject *wrap_matchMode(PyObject *self, PyObject *args)
{
  int  dbNum;
  char *pattern;
  int  mode;
  int  maxNum;
  PyObject *item;

  int *inValues;
  int  returnValue;

  int c,n,i;

  if (!(PyArg_ParseTuple(args, "isii", &dbNum, &pattern, &mode, &maxNum)))
    return Py_None;


  returnValue  = matchMode(dbNum,pattern, mode, maxNum);
  return Py_BuildValue( "i" ,returnValue);
  
}

PyObject *wrap_getMatchingMode(PyObject *self, PyObject *args)
{
  int  dbNum;
  char *pattern;
  int  mode;
  int  maxNum;
  PyObject *item;

  int *inValues;

  int* returnArray; 

  std::vector<int>  returnValues;
  int c,n,i;

  PyObject *list;
  int listSize = -1;
 
  if (!(PyArg_ParseTuple(args, "isii", &dbNum, &pattern, &mode, &maxNum)))
    return Py_None;


  returnArray  = getMatchingMode(dbNum,pattern, mode, maxNum);
  listSize     = getSizeLastMatch();
  list         = PyList_New(listSize);

  for (c=0;c<listSize;c++)
  {
      PyList_SetItem(list, c, PyInt_FromLong (returnArray[c]));     
  }
  delete []returnArray;
  return list;
}

PyObject *wrap_setCurrentMatchSet(PyObject *self, PyObject *args)
{
  int  dbNum;
  PyObject *int_seq = NULL, *int_iter = NULL;
  PyObject *item;

  int *inValues;
  int c,n,i;

  if (!(PyArg_ParseTuple(args, "iO", &dbNum, &int_seq)))
    return Py_None;

  if(!(PyList_Check(int_seq)))
    return Py_None;
  
  if (PyList_Sort(int_seq)==0)
  {
    n = PyList_Size(int_seq);

    inValues = new int[n];

    for (i = 0; i < n; i++) 
    {
      item = PyList_GetItem(int_seq, i); /* Can't fail */
      if (!PyInt_Check(item))
      {/* Skip non-integers */
	  cerr << "wrapper:NON INT \n";  
	  continue;
      }
      inValues[i] = (int) PyInt_AS_LONG(item);
    }  
    setCurrentMatchSet(dbNum, inValues);
    setCurrentMatchSetSize(dbNum, n);
    return PyInt_FromLong(1);
  }
  else
  {
    return Py_None;
  }
}

PyObject *wrap_getCurrentMatchSet(PyObject *self, PyObject *args)
{
   int  dbNum;
   int* returnArray; 

   int*  returnValues;
   int c,n;
   std::vector<int>::iterator theIterator;

   PyObject *list;
   int listSize = -1;

   if (!(PyArg_ParseTuple(args, "i", &dbNum)))
    return Py_None;

  
   returnValues = getCurrentMatchSet(dbNum);
   n = getCurrentMatchSetSize(dbNum);
   list = PyList_New(n);
   for (c=0;c<n;c++)
   {
     PyList_SetItem(list, c, PyInt_FromLong (returnValues[c]));     
   }

   return list;
}

PyObject *wrap_getMatching(PyObject *self, PyObject *args)
{
  int  dbNum;
  char *pattern;

  int* returnArray; 

  std::vector<int>  returnValues;
  int c;
  std::vector<int>::iterator theIterator;

  PyObject *list;
  int listSize = -1;
 
  if (!(PyArg_ParseTuple(args, "is", &dbNum, &pattern)))
    return Py_None;

  returnValues = getMatching(dbNum,pattern, -1, 0);

  listSize = returnValues.size();
  list = PyList_New(listSize);
  returnArray = new int[listSize];

  copy(returnValues.begin(), returnValues.end(), returnArray);

  for (c=0;c<listSize;c++)
    {
      PyList_SetItem(list, c, PyInt_FromLong (returnArray[c]));     
    }
  delete returnArray;
  return list;
  
}

static PyMethodDef OBGrepMethods[] = 
{ 
  { "loadDB", wrap_loadDB, METH_VARARGS }, 
  { "match", wrap_match, METH_VARARGS }, 
  { "matchMode", wrap_matchMode, METH_VARARGS }, 
  { "getNumEntries", wrap_getNumEntries, METH_VARARGS }, 
  { "getEntryString", wrap_getEntryString, METH_VARARGS }, 
  { "getMatching", wrap_getMatching, METH_VARARGS }, 
  { "getMatchingMode", wrap_getMatchingMode, METH_VARARGS }, 
  { "setCurrentMatchSet", wrap_setCurrentMatchSet, METH_VARARGS }, 
  { "getCurrentMatchSet", wrap_getCurrentMatchSet, METH_VARARGS }, 
  { "printEntry", wrap_printEntry, METH_VARARGS }, 
  { "printDB", wrap_printDB, METH_VARARGS }, 
  { NULL, NULL }
}; 

extern "C" DL_EXPORT(void) initOBGrep(void)
{ 
  PyObject *m = Py_InitModule("OBGrep", OBGrepMethods); 
}

int main(int argc, char *argv[])
{
  Py_SetProgramName(argv[0]);
  Py_Initialize();
  initOBGrep();  
}



